<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
// no direct access.
defined('_JEXEC') or die;

use Sellacious\Cache\Exception\CacheMissingException;
use Sellacious\Cache\Reader\ProductsCacheReader;
use Sellacious\Config\Config;
use Sellacious\Config\ConfigHelper;
use Sellacious\Database\Table\Field\SelectField;
use Sellacious\Database\Table\Filter\FilterElement;
use Sellacious\Database\Table\Filter\FilterSet;
use Sellacious\Database\Table\Sort\TableSort;
use Sellacious\Event\EventHelper;
use Sellacious\Price\PriceHelper;

/**
 * Methods supporting a list of Products
 *
 * @since   1.0.0
 */
class SellaciousModelProducts extends SellaciousModelList
{
	/**
	 * The handler to load products records from cache db
	 *
	 * @var   ProductsCacheReader
	 *
	 * @since   2.0.0
	 */
	protected $loader;

	/**
	 * The configuration values for component
	 *
	 * @var   Config
	 *
	 * @since   2.0.0
	 */
	protected $config;

	/**
	 * Constructor
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JControllerLegacy
	 *
	 * @throws  CacheMissingException|Exception
	 *
	 * @since   1.0.0
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id',
				'product_id',
				'title',
				'product_title',
			);
		}

		parent::__construct($config);

		$this->loader = new ProductsCacheReader;
		$this->config = ConfigHelper::getInstance('com_sellacious');
	}

	/**
	 * Method to auto-populate the model state
	 *
	 * This method should only be called once per instantiation and is designed
	 * to be called on the first call to the getState() method unless the model
	 * configuration flag to ignore the request is set
	 *
	 * Note: Calling getState in this method will result in recursion
	 *
	 * @param   string  $ordering   An optional ordering field
	 * @param   string  $direction  An optional direction (asc|desc)
	 *
	 * @return  void
	 *
	 * @since   1.0.0
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		$catId     = $this->app->input->getInt('category_id');
		$prevCatId = $this->app->getUserState('com_sellacious.products.filter.category_id');

		// If product category is changed, then reset filters
		if ($catId && $catId != $prevCatId)
		{
			$this->app->setUserState($this->context . '.filter', array());
		}

		parent::populateState($ordering, $direction);

		$value = $this->app->input->getString('q');

		$this->state->set('filter.search', $value);
		$this->app->setUserState('com_sellacious.products.filter.search', $value);

		if ($catId)
		{
			$this->state->set('filter.category_id', $catId);
			$this->app->setUserState('com_sellacious.products.filter.category_id', $catId);
		}

		if ($value = $this->app->input->getInt('id'))
		{
			$this->state->set('store.id', $value);
		}

		$value = $this->app->input->getString('custom_ordering');

		if ($value)
		{
			$this->state->set('list.custom_ordering', $value);
		}

		$arguments = array(
			'context' => 'com_sellacious.products',
			'loader'  => &$this->loader,
			'state'   => &$this->state,
		);

		EventHelper::trigger('onPopulateState', $arguments);
	}

	/**
	 * Method to get an array of data items
	 *
	 * @return  stdClass[]|bool  An array of data items on success, false on failure
	 *
	 * @since   2.0.0
	 */
	public function getItems()
	{
		$state = $this->getState();
		$store = serialize($state->getProperties());

		if (isset($this->cache[$store]))
		{
			return $this->cache[$store];
		}

		try
		{
			$key    = md5($store);
			$jCache = JCache::getInstance('output', array('caching' => true));
			$items  = $jCache->get($key, 'product_cache');

			if ($items)
			{
				return $this->cache[$store] = $items;
			}

			$this->loader->filterValue('product_active', 1);
			$this->loader->filterValue('seller_active', 1);
			$this->loader->filterValue('listing_active', 1);
			$this->loader->filterValue('is_selling', 1);

			$or = new FilterSet('or');

			$or->addElement(new FilterElement('=', 'variant_id', 0));
			$or->addElement(new FilterElement('=', 'variant_active', 1));

			$this->loader->addFilter($or);

			$arguments = array(
				'context' => 'com_sellacious.products',
				'loader'  => $this->loader,
				'state'   => $state,
			);

			$this->processQuery();

			EventHelper::trigger('onFetchCacheItems', $arguments);

			$selects   = $this->loader->getSelects();
			$orderings = $this->loader->getSortOrders();

			$this->loader->addSelect(new SelectField('batch_id'), true);
			$this->loader->addSelect(new SelectField('is_visible', 'is_visible', 'MAX'));
			$this->loader->addGroupBy('batch_id');

			$items      = array();
			$candidates = $this->loader->getItems($this->getStart(), $state->get('list.limit', 20));

			if ($candidates)
			{
				$loader = new ProductsCacheReader;

				foreach ($selects as $select)
				{
					$loader->addSelect($select);
				}

				foreach ($orderings as $ordering)
				{
					$loader->addSort($ordering);
				}

				$or = new FilterSet('or');

				foreach ($candidates as $candidate)
				{
					$and = new FilterSet('and');

					$and->addElement(new FilterElement('=', 'batch_id', $candidate->batch_id));
					$and->addElement(new FilterElement('=', 'is_visible', $candidate->is_visible));

					$or->addElement($and);
				}

				$loader->addFilter($or);

				$items = $loader->getItems();

				$items = $this->processList($items);
			}

			$jCache->store($items, $key, 'product_cache');

			$this->cache[$store] = $items;
		}
		catch (Exception $e)
		{
			$this->setError($e->getMessage());

			return false;
		}

		return $this->cache[$store];
	}

	/**
	 * Returns a record count for the query
	 *
	 * @param   JDatabaseQuery  $query  The query
	 *
	 * @return  int  Number of rows for query
	 *
	 * @since   2.0.0
	 */
	protected function _getListCount($query)
	{
		return (int) $this->loader->getTotal();
	}

	/**
	 * Method to process the list query with filters and sorting
	 *
	 * @return  void
	 *
	 * @throws  Exception
	 *
	 * @since   2.0.0
	 */
	protected function processQuery()
	{
		$tag = JFactory::getLanguage()->getTag();

		$this->loader->filterValue('language', array($tag, '*', ''));

		$catId = $this->getState('filter.category_id', 1);

		if ($catId > 1)
		{
			$this->loader->addFilter(new FilterElement('set_contains', 'category_id_ext', (int) $catId));
		}

		$storeId = $this->getState('store.id', 0);

		if ($storeId > 0)
		{
			// This filter exists in Filter module (store filter) as well but its duplicated here
			// because store.id is fetched from "store" model state but store filter is fetched from user state
			$this->loader->filterValue('seller_uid', (int) $storeId);
		}

		$this->loader->addFilter(new FilterElement('>', 'is_visible', 0));

		// Hide zero price to be done using cache column 'price_valid'

		// The no-stock items are always at end
		$this->loader->addSort(new TableSort('no_stock', TableSort::SORT_ASC));

		$ordering = $this->getState('list.custom_ordering');

		switch ($ordering)
		{
			case 'newest':
				$this->loader->addSort(new TableSort('listing_start', TableSort::SORT_DESC));
				break;
			case 'order_max':
				$this->loader->addSort(new TableSort('order_units', TableSort::SORT_DESC));
				break;
			case 'rating_max':
				$this->loader->addSort(new TableSort('product_rating', TableSort::SORT_DESC));
				break;
			case 'price_min':
				$this->loader->addSort(new TableSort('sales_price_fx', TableSort::SORT_ASC));
				break;
			case 'price_max':
				$this->loader->addSort(new TableSort('sales_price_fx', TableSort::SORT_DESC));
				break;
		}

		$this->loader->addSort(new TableSort('is_visible', TableSort::SORT_DESC));
	}

	/**
	 * Pre-process loaded list before returning if needed
	 *
	 * @param   stdClass[]  $items  List loaded from the listQuery
	 *
	 * @return  stdClass[]
	 *
	 * @throws  Exception
	 *
	 * @since   1.3.0
	 */
	protected function processList($items)
	{
		foreach ($items as &$item)
		{
			$item->category         = is_string($item->category) ? json_decode($item->category, true) : (array) $item->category;
			$item->category_ext     = is_string($item->category_ext) ? json_decode($item->category_ext, true) : (array) $item->category_ext;
			$item->spl_category     = is_string($item->spl_category) ? json_decode($item->spl_category, true) : (array) $item->spl_category;
			$item->product_rating   = is_string($item->product_rating) ? json_decode($item->product_rating) : $item->product_rating;
			$item->product_features = is_string($item->product_features) ? json_decode($item->product_features) : $item->product_features;

			$handler = PriceHelper::getHandler($item->pricing_type);

			$handler->processCacheProduct($item);
		}

		return $items;
	}
}
