<?php
/**
 * @version     2.0.0
 * @package     sellacious
 *
 * @copyright   Copyright (C) 2012-2020 Bhartiy Web Technologies. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 * @author      Izhar Aazmi <info@bhartiy.com> - http://www.bhartiy.com
 */
// no direct access.
defined('_JEXEC') or die;

use Sellacious\Cache\Reader\ProductsCacheReader;
use Sellacious\Database\Table\Filter\FilterElement;
use Sellacious\Database\Table\Filter\FilterSet;
use Sellacious\Price\PriceHelper;

/**
 * Methods supporting a list of Products
 *
 * @since   1.2.0
 */
class SellaciousModelWishlist extends SellaciousModelList
{
	/**
	 * The handler to load products records from cache db
	 *
	 * @var   ProductsCacheReader
	 *
	 * @since   2.0.0
	 */
	protected $loader;

	/**
	 * Constructor
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @see     JControllerLegacy
	 *
	 * @throws  Exception
	 *
	 * @since   1.2.0
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'title', 'a.title',
				'alias', 'a.alias',
				'state', 'a.state',
			);
		}

		$this->loader = new ProductsCacheReader;

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 *
	 * This method should only be called once per instantiation and is designed
	 * to be called on the first call to the getState() method unless the model
	 * configuration flag to ignore the request is set
	 *
	 * Note: Calling getState in this method will result in recursion
	 *
	 * @param   string  $ordering   An optional ordering field.
	 * @param   string  $direction  An optional direction (asc|desc)
	 *
	 * @return  void
	 *
	 * @since   12.2
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		parent::populateState($ordering, $direction);

		$uid  = $this->app->input->get('user_id');
		$user = JFactory::getUser($uid);

		$this->state->set('wishlist.user_id', $user->id);
	}

	public function getItems()
	{
		$store = $this->getStoreId();

		if (isset($this->cache[$store]))
		{
			return $this->cache[$store];
		}

		try
		{
			$db     = $this->getDbo();
			$query  = $db->getQuery(true);
			$userId = $this->state->get('wishlist.user_id');

			$query->select('id, user_id, product_id, variant_id, seller_uid, created')
				->from($db->qn('#__sellacious_wishlist', 'a'))
				->where('a.user_id = ' . (int) $userId);

			$iterator = $db->setQuery($query)->getIterator();

			if ($iterator->count())
			{
				$cond = new FilterSet('or');

				$multi_variant = $this->helper->config->get('multi_variant');
				$multi_seller  = $this->helper->config->get('multi_seller');

				foreach ($iterator as $row)
				{
					$and = new FilterSet('and');

					$and->addElement(new FilterElement('=', 'product_id', (int) $row->product_id));

					if ($multi_variant)
					{
						$and->addElement(new FilterElement('=', 'variant_id', (int) $row->variant_id));
					}

					if ($multi_seller)
					{
						$and->addElement(new FilterElement('=', 'seller_uid', (int) $row->seller_uid));
					}

					$cond->addElement($and);
				}

				$this->loader->addFilter($cond);
			}
			else
			{
				$this->loader->fallacy();
			}

			$items = $this->loader->getItems($this->getStart(), $this->getTotal());
			$items = $this->processList($items);

			$this->cache[$store] = $items;
		}
		catch (Exception $e)
		{
			$this->setError($e->getMessage());

			return false;
		}

		return $this->cache[$store];
	}

	/**
	 * Pre-process loaded list before returning if needed
	 *
	 * @param   stdClass[]  $items  List loaded from the listQuery
	 *
	 * @return  stdClass[]
	 *
	 * @throws  Exception
	 *
	 * @since   1.3.0
	 */
	protected function processList($items)
	{
		foreach ($items as &$item)
		{
			$item->category         = is_string($item->category) ? json_decode($item->category, true) : (array) $item->category;
			$item->category_ext     = is_string($item->category_ext) ? json_decode($item->category_ext, true) : (array) $item->category_ext;
			$item->spl_category     = is_string($item->spl_category) ? json_decode($item->spl_category, true) : (array) $item->spl_category;
			$item->product_rating   = is_string($item->product_rating) ? json_decode($item->product_rating) : $item->product_rating;
			$item->product_features = is_string($item->product_features) ? json_decode($item->product_features) : $item->product_features;

			$handler = PriceHelper::getHandler($item->pricing_type);

			$handler->processCacheProduct($item);
		}

		return $items;
	}

	/**
	 * Returns a record count for the query
	 *
	 * @param   JDatabaseQuery  $query  The query
	 *
	 * @return  int  Number of rows for query
	 *
	 * @since   2.0.0
	 */
	protected function _getListCount($query)
	{
		return (int) $this->loader->getTotal();
	}
}
